package org.openxma.dsl.pom.scoping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.pom.model.AttachmentPosition;
import org.openxma.dsl.pom.model.Command;
import org.openxma.dsl.pom.model.Component;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.CustomizationOfComposite;
import org.openxma.dsl.pom.model.CustomizationOfGuiElement;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.StringDefinition;
import org.openxma.dsl.pom.model.XmadslPage;

import at.spardat.xma.guidesign.PageComposite;
import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.XMAFunction;

/**
 * This class collects defined elements in the Pom. It is intended to be used
 * for all kind of element collections, e.g. by the Pom ScopeProvider.
 */
public class PomElementCollector {

	private final static String CONTAINING_PAGE_PROPERTY = "containingPage";

	private static PomElementCollector instance;

	protected PomElementCollector() {
	}

	/**
	 * Get the instance of this class.
	 *
	 * @return
	 */
	public static PomElementCollector getInstance() {
		if (instance == null) {
			instance = new PomElementCollector();
		}
		return instance;
	}

	/**
	 * Collect all Commands which are accessible from a Page or a Component. Commands
	 * which are hidden be other ones are not contained.
	 *
	 * @param page
	 * @return
	 */
	public static Collection<Command> getAccessibleCommands(Page page, Component containingComponent) {
		HashMap<String,Command> commands = new HashMap<String,Command>();
		for (Command cmd : containingComponent.getCommands()) {
			commands.put(cmd.getName(),cmd);
		}
		if (page != null) {
			getAccessibleCommands(page,commands);

		}
		return commands.values();
	}
	protected static void getAccessibleCommands(Page page, HashMap<String,Command> commands) {
		Page superPage = getSuperPage(page);
		if (superPage != null) {
			getAccessibleCommands(superPage,commands);
		}
		Page containingPage = getContainingPage(page.eContainer());
		if (containingPage != null) {
			getAccessibleCommands(containingPage,commands);
		}
		for (Command cmd : page.getCommands()) {
			commands.put(cmd.getName(),cmd);
		}
	}

	public static Collection<Page> getAccessiblePages(EObject obj) {
		Collection<Page> pages = new ArrayList<Page>();
		Page p = getContainingPage(obj);
		while (p != null) {
			pages.add(p);
			p = getContainingPage(p.eContainer());
		}
		return pages;
	}

	/**
	 * Collect all XMAFunctions which are accessible from a XMAPage or a
	 * XMAComponent. In case that a XMAFunction is not visible because it
	 * is hidden by an other XMAFunction with the same name the invisible
	 * one is not returned.
	 *
	 * @param page
	 * @param containingComponent
	 * @return
	 */
	public Collection<XMAFunction> getAccessibleXMAFunctions(XMAPage page, XMAComponent containingComponent) {
		HashMap<String,XMAFunction> functions = new HashMap<String,XMAFunction>();
		Iterator it = containingComponent.getFunctions().iterator();
		while (it.hasNext()) {
			XMAFunction func = (XMAFunction)it.next();
			functions.put(func.getNamFunction(),func);
		}
		if (page != null) {
			getAccessibleXMAFunctions(page,functions);
		}
		return functions.values();
	}
	protected void getAccessibleXMAFunctions(XMAPage page, HashMap<String,XMAFunction> functions) {
		Iterator it = page.getFunctions().iterator();
		XMAPage containingPage = getContainingXMAPage(page.eContainer());
		if (containingPage != null) {
			getAccessibleXMAFunctions(containingPage,functions);
		}
		while (it.hasNext()) {
			XMAFunction func = (XMAFunction)it.next();
			functions.put(func.getNamFunction(),func);
		}
	}

	public Collection<AttachmentPosition> getAccessibleAttachmentPositions(Page page) {
		return getAccessibleAttachmentPositions(page,new HashSet<AttachmentPosition>());
	}
	protected Collection<AttachmentPosition> getAccessibleAttachmentPositions(Page page, HashSet<AttachmentPosition> positions) {
		if (page.getDefinitions() != null) {
			positions.addAll(page.getDefinitions().getPositions());
		}
		Page superPage = getSuperPage(page);
		if (superPage != null) {
			getAccessibleAttachmentPositions(superPage,positions);
		}
		Page containingPage = getContainingPage(page.eContainer());
		if (containingPage != null) {
			getAccessibleAttachmentPositions(containingPage,positions);
		} else {
			Component component = getContainingComponent(page);
			if (component.getDefinitions() != null) {
				positions.addAll(component.getDefinitions().getPositions());
			}
		}
		return positions;
	}



	public Collection<StringDefinition> getAccessibleStringDefinitions(Page page) {
		return getAccessibleStringDefinitions(page,new HashSet<StringDefinition>());
	}
	protected Collection<StringDefinition> getAccessibleStringDefinitions(Page page, HashSet<StringDefinition> positions) {
		if (page.getDefinitions() != null) {
			positions.addAll(page.getDefinitions().getStrings());
		}
		Page superPage = getSuperPage(page);
		if (superPage != null) {
			getAccessibleStringDefinitions(superPage,positions);
		}
		Page containingPage = getContainingPage(page.eContainer());
		if (containingPage != null) {
			getAccessibleStringDefinitions(containingPage,positions);
		} else {
			Component component = getContainingComponent(page);
			if (component.getDefinitions() != null) {
				positions.addAll(component.getDefinitions().getStrings());
			}
		}
		return positions;
	}

	/**
	 * Get the containing Component of a Pom object.
	 *
	 * @param eObject
	 * @return
	 */
	public static Component getContainingComponent(EObject eObject) {
		if (null == eObject || eObject instanceof Component) {
			return (Component) eObject;
		} else if (eObject instanceof CustomizationOfGuiElement) {
			getContainingComponent(((CustomizationOfGuiElement) eObject).getPage());
		}
		return getContainingComponent(eObject.eContainer());
	}

	/**
	 * Get the containing Page of a Pom object.
	 *
	 * @param eObject
	 * @return
	 */
	public static Page getContainingPage(EObject eObject) {
		if (eObject instanceof Page) {
			return (Page) eObject;
		} else if (eObject instanceof CustomizationOfGuiElement) {
			return ((CustomizationOfGuiElement) eObject).getPage();
		}
		if (eObject == null) return null;
		return eObject.eContainer() != null ? getContainingPage(eObject.eContainer()) : null;
	}

	/**
	 * Get the XMAPage which contains the object
	 *
	 * @param obj
	 * @return
	 */
	public static XMAPage getContainingXMAPage(EObject obj) {
		XMAPage xmaPage = (XMAPage)EObjectPropertiesAdapter.getProperty(obj,CONTAINING_PAGE_PROPERTY);
		if (xmaPage == null) {
			// Remark: this.parentcomp does not work in all situations because parentcomp does not always
			// has a value (e.g. in case of a XMAScrolledComposite)
			if (obj instanceof PageComposite) {
				xmaPage = ((PageComposite)obj).getPage();
			} else if (obj.eContainer() != null && obj.eContainer() instanceof XMAComponent == false){
				xmaPage = getContainingXMAPage(obj.eContainer());
			}
			if (xmaPage != null) {
				EObjectPropertiesAdapter.setProperty(obj,CONTAINING_PAGE_PROPERTY,xmaPage);
			}
		}
		return xmaPage;
	}

	/**
	 * Get the super Page of a Page.
	 *
	 * @param page
	 * @return
	 */
	public static Page getSuperPage(Page page) {
		if (page instanceof XmadslPage) {
			XmadslPage xmadslPage= (XmadslPage)page;
			return xmadslPage.getTemplate();
		}
		return null;
	}

	/**
	 * Get the container of a object which have ComposeData.
	 *
	 * @param eObject
	 * @return
	 */
	public static Composite getContainerWithComposeData(EObject eObject) {
		EObject obj = eObject.eContainer();
		if (obj == null) {
			return null;
		}
		if (obj instanceof Composite) {
			return (Composite) obj;
		}
		if (obj instanceof CustomizationOfGuiElement) {
			if (obj instanceof CustomizationOfComposite) {
				return ((CustomizationOfComposite)obj).getComposite();
			}
		}
		return getContainerWithComposeData(obj);
	}
	
	public static List<EObject> getAllElementsOfXMAPage(XMAPage xmaPage) {
		List<EObject> elementsOfPage = PomDslScopeProvider.getNsElementsInRange(xmaPage,null);		
		return elementsOfPage;
	}
	
	public static <T> List<T> getElementsOfXMAPage(XMAPage xmaPage, Class<T> clazz) {
		List<EObject> elementsOfPage = getAllElementsOfXMAPage(xmaPage);
		List<T> widgetsOfPage = EcoreUtil2.typeSelect(elementsOfPage,clazz);				
		return widgetsOfPage;
	}		
	
	public static List<XMAWidget> getWidgetsOfXMAPage(XMAPage xmaPage) {
		List<XMAWidget> widgetsOfPage = getElementsOfXMAPage(xmaPage,XMAWidget.class);
		return widgetsOfPage;
	}

}
